/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalEnvelope.h"
#include "HvSignalLine.h"
#include "HvControlBinop.h"
#include "HvControlCast.h"
#include "HvControlVar.h"
#include "HvControlIf.h"
#include "HvSignalVar.h"
#include "HvMath.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_v0FHcFAf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_BIvRqFpB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_o19i4yQ6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_GTaVEnkW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_jNEmeZWB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_mGq4Whv8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_dhRnDHuQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_CWrg1MCF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_J7FFehrO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_xUO7RF5y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_uDVFX2NV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_aLJe4ueS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_VHekoiob_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_tfW8gLCD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_cW7ehYff_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_CzWLwKsA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_DVpB0Vj6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_6qxmxsEP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_yCwRtEeg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_QSJTtV36_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_0yo9aTyR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_VTAk8ugr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_J8Do6ZZC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_I1iVQiwv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_kYGUfROW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_mp9iIQma_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_RxVVaN9s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1pXEkTwf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_qynH0Jew_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_uSoNd25V_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_fe9ZudMq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_HEGtmD5E_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_XQ39EaGz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_CfKvZGtz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_ixlOKYyi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_sTscIAgG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_v9OSWDF6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_0QlA7XZC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_C6osHlOp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ZI28cXXp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_RSjNsJeQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_VTwZnfC5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_eqORbFPP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_zcBPQcWF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ceCs8Qo7_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_9vddG7Cv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ux27TDuJ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_jINNnU5w_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Cx1jEt0J_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_73CeFyMG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_2w0tqVpW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_eAukDP7U_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_qdOIzW5y_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_O4dYmP3U_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_yhMka39e_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_c8h3j4ON_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_JoXkN5Vi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_HdD0ECIz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_T1OUdISM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_EPUPqVPv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_gRHBvmun_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Xrjn4DAu_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_mEjd6GgU;
  SignalEnvelope sEnv_6qxmxsEP;
  SignalEnvelope sEnv_yCwRtEeg;
  SignalEnvelope sEnv_ixlOKYyi;
  SignalEnvelope sEnv_sTscIAgG;
  SignalLine sLine_JfXEgJwb;
  SignalLine sLine_OXaiOwgL;
  ControlBinop cBinop_v0FHcFAf;
  ControlBinop cBinop_BIvRqFpB;
  ControlBinop cBinop_o19i4yQ6;
  ControlBinop cBinop_GTaVEnkW;
  ControlBinop cBinop_dhRnDHuQ;
  ControlBinop cBinop_J7FFehrO;
  ControlBinop cBinop_xUO7RF5y;
  ControlBinop cBinop_uDVFX2NV;
  ControlVar cVar_aLJe4ueS;
  ControlVar cVar_tfW8gLCD;
  SignalVarf sVarf_NUTmcqlv;
  ControlVar cVar_cW7ehYff;
  ControlIf cIf_CzWLwKsA;
  ControlIf cIf_DVpB0Vj6;
  ControlBinop cBinop_QSJTtV36;
  ControlBinop cBinop_VTAk8ugr;
  ControlBinop cBinop_J8Do6ZZC;
  ControlBinop cBinop_mp9iIQma;
  ControlBinop cBinop_1pXEkTwf;
  SignalVarf sVarf_WI2GZVTG;
  ControlVar cVar_HEGtmD5E;
  ControlIf cIf_XQ39EaGz;
  ControlIf cIf_CfKvZGtz;
  ControlBinop cBinop_v9OSWDF6;
  ControlBinop cBinop_C6osHlOp;
  ControlBinop cBinop_ZI28cXXp;
  ControlBinop cBinop_eqORbFPP;
  ControlBinop cBinop_ceCs8Qo7;
  ControlVar cVar_Cx1jEt0J;
  ControlBinop cBinop_73CeFyMG;
  SignalVarf sVarf_cKaZS1aG;
  SignalVarf sVarf_em8kSdUP;
  SignalVarf sVarf_8mRrAqwK;
  SignalVarf sVarf_pVyiRgaM;
  ControlBinop cBinop_O4dYmP3U;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
